#ifndef __SSD1606_H__
#define __SSD1606_H__

class SSD1606
{
    public:
        SSD1606(PinName cs, PinName rs, PinName dc, PinName clk, PinName busy, PinName data);
        void reset();
        void initialize();
        void update();
        void off();
        void on();
        void sleep();
        void wake();
        void set_inverse();
        void set_lut();
        void set_command_between_images();
        void set_display_start_line(unsigned char val);
        void nop();
        void set_charge_pump_enable(unsigned char enable);
        void clear();
        void set_pixel(int x, int y);
        void clear_pixel(int x, int y);
        void set_font(unsigned char *font, unsigned char width);
        void set_double_height_text(unsigned int double_height);
        void putc(unsigned char c);
        void printf(const char *format,...);
        unsigned char is_busy(void);
        void rom_image(const unsigned char *bitmap);
        
    private:
        SPI _spi;
        DigitalOut _cs, _reset, _dc;
        DigitalIn _busy;
        unsigned char _screen[3096];
    int _cursor_x, _cursor_y;

    void _send_command(unsigned char code);
    void _send_data(unsigned char value);
    
    unsigned char *_console_font_data;  
    unsigned int _console_font_width;
    unsigned int _double_height_text;   
};

/*
    The width/height parameters depend on different vendor. The max is 180x128
    http://www.solomon-systech.com.cn/cn/product/advanced-display/bistable-display-driver-controller/
 */
#define SSD1606_LCDWIDTH 172
#define SSD1606_LCDHEIGHT 72

#endif
